/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "sqlschema.h"

// Sample application which shows how to access data via SQL
// queries.  Shows both the traditional form of accessing individual
// attribute values (columns) as well as getting OIDs which map to
// Objetive-C instances.  Database must be loaded before running this
// application; use the supplied sqldata.sql SQL script.
//
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname, *dbNamespace;
  
  if (argc < 3) {
    printf("Usage: %s <host> <database>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  dbNamespace = @"examples.objc_examples.chap_9";

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chap_9"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init: @"" : dbNamespace];
    db = [(MtDatabase *)[MtDatabase alloc] init : hostname : dbname andFactory: factory];

    [db open];

    [db startVersionAccess];

    // Set the SQL CURRENT_NAMESPACE to 'dbNamespace' so there is
    // no need to use the full qualified names to acces the schema objects
    [db setSqlCurrentNamespace:dbNamespace];

    printf("Querying: %s\n", [[db description] UTF8String]);
    printf("Sql Default Namespace: %s\n", [[db sqlCurrentNamespace] UTF8String]);

#if __has_feature(objc_arc)
    @autoreleasepool {
#endif

      // create a statement, execute some sql and iterate through row/cols
      MtStatement* stmt = [[MtStatement alloc] init:db];
      NSString *query = @"SELECT e.name, e.id, e.boss.name FROM Employee e WHERE e.id > 2";
      printf("Sql Default Namespace: %s\n", [[db sqlCurrentNamespace] UTF8String]);
      printf("Query: %s\n", [query UTF8String]); 

      MtResultSet* res = [stmt executeQuery:query];

      unsigned int i;
      // list names and types
      // column index is 1-based
      for (i = 1; i <= [res getColumnCount]; i++) {
	printf("column #%d '%s' type is: %s\n", i, 
	       [[res getColumnName:i] UTF8String], 
	       [[res getColumnTypeName:i] UTF8String]);
      }
      // since we know that we asked for string and integer, we can dump
      //  all the results
      // use the next method to move through rows
      while ([res next]) {
	NSString* bnam = [res getString:3];
	BOOL isnull = [res wasNull];
	printf("Employee name: %s, id: %d, boss: %s (%d)\n",
	       [[res getString:1] UTF8String], 
	       [res getInteger:2], 
	       (!isnull ? [bnam UTF8String]  : "[no boss]"), 
	       isnull);
      }

      // always remember to close the statement when done
      [stmt close];

      // to get a list of objects, use the Ref operator

      MtStatement* stmt2 = [[MtStatement alloc] init:db];
      NSString *query2 = @"SELECT Ref(Employee), Ref(boss) FROM Employee WHERE id > 2;";
      printf("Sql Default Namespace: %s\n", [[db sqlCurrentNamespace] UTF8String]);
      printf("Query: %s\n", [query2 UTF8String]); 

      MtResultSet* res2 = [stmt2 executeQuery:query2];

      // list names and types
      // column index is 1-based
      for (i = 1; i <= [res2 getColumnCount]; i++) {
	printf("column #%d '%s' type is: %s\n", i, 
	       [[res2 getColumnName:i] UTF8String], 
	       [[res2 getColumnTypeName:i] UTF8String]);
      }
      Employee *e;
      Manager *m;
      // still use the ResultSet, but get the object and coerce to the
      // class we expect
      while ([res2 next]) {
	e = (Employee *)[res2 getMtObject:1];
	m = (Manager *)[res2 getMtObject:2];
	BOOL isnull = [res2 wasNull];
	printf("Employee name: %s, id: %d, boss: %s (isnull=%d)\n",
	       [[e name] UTF8String], 
	       [e id], 
	       (m ? [[m name] UTF8String] : "[no boss]"), 
	       isnull);
      }

      // always remember to close the statement when done
      [stmt2 close];

#if __has_feature(objc_arc)
    } // @autoreleasepool
#endif

    [db endVersionAccess];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
