/*
 *  Copyright (c) 2012 Matisse Software Inc. All Rights Reserved.
 *
 *  This file (in both binary and source code form) is subject to the
 *  Supplemental License Terms governing use, modification and redistribution
 *  of Sample Code accompanying the Matisse(R) software.
 *
 */

#import <Foundation/Foundation.h>

// Matisse Objective-C header
#import "matisseObjC.h"

#import "Person.h"
#import "Employee.h"

// A simple example which shows how to use generated code for user classes.
int main(int argc, char **argv)
{
  MtDatabase* db;
  NSString *hostname, *dbname, *firstName, *lastName;
  
  if (argc < 5) {
    printf("Usage: %s <host> <database> <firstName> <lastName>\n", argv[0]);
    return -1;
  }
#if !__has_feature(objc_arc)
  NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
#else
  @autoreleasepool {
#endif

  hostname = [NSString stringWithCString:argv[1] encoding:NSUTF8StringEncoding]; 
  dbname = [NSString stringWithCString:argv[2] encoding:NSUTF8StringEncoding];

  firstName = [NSString stringWithCString:argv[3] encoding:NSUTF8StringEncoding];
  lastName = [NSString stringWithCString:argv[4] encoding:NSUTF8StringEncoding];

  @try {

    // The third argument to MtDatabase init is given so that the connection 
    // object can find the mapping between the Objective-C class Person and 
    // the schema class Person defined in the "examples.objc_examples.chaps_6_7_8"
    // namespace.
    //
    MtDynamicObjectFactory *factory = [[MtDynamicObjectFactory alloc] init:@"" :@"examples.objc_examples.chaps_6_7_8"];
    db = [(MtDatabase *)[MtDatabase alloc] init:hostname :dbname andFactory:factory];

    [db open];

    [db startTransaction];
    printf("\nCreating objects...\n");
    // create a new Person
    Person* p1 = [[Person alloc] init:db];
    // modify attributes
    [p1 setFirstName:@"John"];
    [p1 setLastName:@"Jones"];

    Person* p2 = [[Person alloc] init:db];
    // modify attributes
    [p2 setFirstName:@"Fred"];
    [p2 setLastName:@"Jones"];

    Person* p3 = [[Person alloc] init:db];
    // modify attributes
    [p3 setFirstName:@"John"];
    [p3 setLastName:@"Murray"];

    Person* p4 = [[Person alloc] init:db];
    // modify attributes
    [p4 setFirstName:@"Fred"];
    [p4 setLastName:@"Flintstone"];

    [db commit];

    [db startVersionAccess];
 
    // just show the instances available
    Person* x;
    MtObjectEnumerator* piter = [Person instanceEnumerator: db];
    while ((x = (Person*)[piter nextObject]) != nil) {
      printf("%s %s\n", 
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String]);
    }
    [piter close];

    printf("\nLooking for : %s %s\n", 
	   [firstName UTF8String],
	   [lastName UTF8String]);
    
    
    // The lookup function must return a Person* to allow for nil
    // to represent no match
    Person *found = [Person lookupPersonName:db :lastName :firstName];
    if (found != nil) {
      printf("found %s %s\n", 
	     [[found firstName] UTF8String],
	     [[found lastName] UTF8String]);
    } else {
      printf("Nobody found\n");
    }

    // instead of searching, open an iterator within the specified
    // criteria; an Index can specify upto 4 criteria and this API
    // would change accordingly (see examples.odl for specification)
    NSString *fromFirstName = @"Fred";
    NSString *fromLastName = @"Jones";
    NSString *toFirstName = @"John";
    NSString *toLastName = @"Murray";
    printf("\nLookup Index from %s %s to %s %s\n", 
	   [fromFirstName UTF8String],
	   [fromLastName UTF8String], 
	   [toFirstName UTF8String],
	   [toLastName UTF8String]);

    //fromFirstName = nil;
    //toFirstName = nil;
    //fromLastName = nil;
    //toLastName = nil;

    MtObjectEnumerator* it = [Person personNameEnumerator:db :fromLastName :fromFirstName :toLastName :toFirstName :nil :MT_DIRECT :MT_MAX_PREFETCHING];
    while ((x = (Person*)[it nextObject]) != nil) {
      printf("%s %s\n", 
	     [[x firstName] UTF8String],
	     [[x lastName] UTF8String]);
    }
    [it close];

    NSArray* key = [NSArray arrayWithObjects:  fromLastName,  fromFirstName, nil];
    //NSArray* key = [NSArray arrayWithObjects:  fromLastName, nil, nil];

    //unsigned int num = [[Person getPersonNameIndex:db] getObjectNumber:fromLastName :nil];
    unsigned int num = [[Person getPersonNameIndex:db] getObjectNumberWithKeys:key :nil];
    printf("\n%d object(s) retrieved\n", num);

    unsigned int count = [[Person getPersonNameIndex:db] getIndexEntriesNumber];
    printf("\n%d entries in the index\n", count);

    [db endVersionAccess];


    [db startTransaction:MT_MAX_TRAN_PRIORITY];
    printf("\nRemoving created objects...\n");
    [p1 remove];
    [p2 remove];
    [p3 remove];
    [p4 remove];

    [db commit];

    [db close];

  }
  @catch (MtException *e) {
    printf("ERROR: %s: (code=%d) %s\n", 
	   [[e name] UTF8String], 
	   [e errorCode], 
	   [[e message] UTF8String] );
    NSLog(@"Stack trace: %@\n", [e callStackSymbols]);
  }
  @finally {
#if !__has_feature(objc_arc)
    [db release];
#endif
  }
#if !__has_feature(objc_arc)
  // free pool
  [pool release];
#else
  } // @autoreleasepool
#endif

  printf("Done\n");

  return 0;
}
